#!/usr/bin/env ipython
# Saves me quite a bit of typing during interactive
# testing sessions.
# Just execute.
import sys
sys.path.append('.')

from stackexchange import *
from stackauth import *
#so = Site(StackOverflow, '1_9Gj-egW0q_k1JaweDG8Q')
so = Site(StackOverflow, 'pXlviKYs*UZIwKLPwJGgpg((')
