import stackexchange
class __SEAPI(str):
    def __call__(self):
        return stackexchange.Site(self)
StackOverflow = __SEAPI('api.stackoverflow.com')
ServerFault = __SEAPI('api.serverfault.com')
SuperUser = __SEAPI('api.superuser.com')
MetaStackOverflow = __SEAPI('api.meta.stackoverflow.com')
WebApplications = __SEAPI('api.webapps.stackexchange.com')
WebApplicationsMeta = __SEAPI('api.meta.webapps.stackexchange.com')
Gaming = __SEAPI('api.gaming.stackexchange.com')
GamingMeta = __SEAPI('api.meta.gaming.stackexchange.com')
Webmasters = __SEAPI('api.webmasters.stackexchange.com')
WebmastersMeta = __SEAPI('api.meta.webmasters.stackexchange.com')
Cooking = __SEAPI('api.cooking.stackexchange.com')
CookingMeta = __SEAPI('api.meta.cooking.stackexchange.com')
GameDevelopment = __SEAPI('api.gamedev.stackexchange.com')
GameDevelopmentMeta = __SEAPI('api.meta.gamedev.stackexchange.com')
Photography = __SEAPI('api.photo.stackexchange.com')
PhotographyMeta = __SEAPI('api.meta.photo.stackexchange.com')
StatisticalAnalysis = __SEAPI('api.stats.stackexchange.com')
StatisticalAnalysisMeta = __SEAPI('api.meta.stats.stackexchange.com')
Mathematics = __SEAPI('api.math.stackexchange.com')
MathematicsMeta = __SEAPI('api.meta.math.stackexchange.com')
HomeImprovement = __SEAPI('api.diy.stackexchange.com')
HomeImprovementMeta = __SEAPI('api.meta.diy.stackexchange.com')
MetaSuperUser = __SEAPI('api.meta.superuser.com')
MetaServerFault = __SEAPI('api.meta.serverfault.com')
GIS = __SEAPI('api.gis.stackexchange.com')
GISMeta = __SEAPI('api.meta.gis.stackexchange.com')
TeXLaTeX = __SEAPI('api.tex.stackexchange.com')
TeXLaTeXMeta = __SEAPI('api.meta.tex.stackexchange.com')
AskUbuntu = __SEAPI('api.askubuntu.com')
AskUbuntuMeta = __SEAPI('api.meta.askubuntu.com')
PersonalFinanceandMoney = __SEAPI('api.money.stackexchange.com')
PersonalFinanceandMoneyMeta = __SEAPI('api.meta.money.stackexchange.com')
EnglishLanguageandUsage = __SEAPI('api.english.stackexchange.com')
EnglishLanguageandUsageMeta = __SEAPI('api.meta.english.stackexchange.com')
StackApps = __SEAPI('api.stackapps.com')
UserExperience = __SEAPI('api.ux.stackexchange.com')
UserExperienceMeta = __SEAPI('api.meta.ux.stackexchange.com')
UnixandLinux = __SEAPI('api.unix.stackexchange.com')
UnixandLinuxMeta = __SEAPI('api.meta.unix.stackexchange.com')
WordPress = __SEAPI('api.wordpress.stackexchange.com')
WordPressMeta = __SEAPI('api.meta.wordpress.stackexchange.com')
TheoreticalComputerScience = __SEAPI('api.cstheory.stackexchange.com')
TheoreticalComputerScienceMeta = __SEAPI('api.meta.cstheory.stackexchange.com')
Apple = __SEAPI('api.apple.stackexchange.com')
AppleMeta = __SEAPI('api.meta.apple.stackexchange.com')
RoleplayingGames = __SEAPI('api.rpg.stackexchange.com')
RoleplayingGamesMeta = __SEAPI('api.meta.rpg.stackexchange.com')
Bicycles = __SEAPI('api.bicycles.stackexchange.com')
BicyclesMeta = __SEAPI('api.meta.bicycles.stackexchange.com')
Programmers = __SEAPI('api.programmers.stackexchange.com')
ProgrammersMeta = __SEAPI('api.meta.programmers.stackexchange.com')
ElectricalEngineering = __SEAPI('api.electronics.stackexchange.com')
ElectricalEngineeringMeta = __SEAPI('api.meta.electronics.stackexchange.com')
AndroidEnthusiasts = __SEAPI('api.android.stackexchange.com')
AndroidEnthusiastsMeta = __SEAPI('api.meta.android.stackexchange.com')
OnStartups = __SEAPI('api.onstartups.stackexchange.com')
OnStartupsMeta = __SEAPI('api.meta.onstartups.stackexchange.com')
BoardandCardGames = __SEAPI('api.boardgames.stackexchange.com')
BoardandCardGamesMeta = __SEAPI('api.meta.boardgames.stackexchange.com')
Physics = __SEAPI('api.physics.stackexchange.com')
PhysicsMeta = __SEAPI('api.meta.physics.stackexchange.com')
Homebrew = __SEAPI('api.homebrew.stackexchange.com')
HomebrewMeta = __SEAPI('api.meta.homebrew.stackexchange.com')
ITSecurity = __SEAPI('api.security.stackexchange.com')
ITSecurityMeta = __SEAPI('api.meta.security.stackexchange.com')
Writers = __SEAPI('api.writers.stackexchange.com')
WritersMeta = __SEAPI('api.meta.writers.stackexchange.com')
AudioVideoProduction = __SEAPI('api.avp.stackexchange.com')
AudioVideoProductionMeta = __SEAPI('api.meta.avp.stackexchange.com')
GraphicDesign = __SEAPI('api.graphicdesign.stackexchange.com')
GraphicDesignMeta = __SEAPI('api.meta.graphicdesign.stackexchange.com')
DatabaseAdministrators = __SEAPI('api.dba.stackexchange.com')
DatabaseAdministratorsMeta = __SEAPI('api.meta.dba.stackexchange.com')
ScienceFictionandFantasy = __SEAPI('api.scifi.stackexchange.com')
ScienceFictionandFantasyMeta = __SEAPI('api.meta.scifi.stackexchange.com')
CodeReview = __SEAPI('api.codereview.stackexchange.com')
CodeReviewMeta = __SEAPI('api.meta.codereview.stackexchange.com')
CodeGolf = __SEAPI('api.codegolf.stackexchange.com')
CodeGolfMeta = __SEAPI('api.meta.codegolf.stackexchange.com')
QuantitativeFinance = __SEAPI('api.quant.stackexchange.com')
QuantitativeFinanceMeta = __SEAPI('api.meta.quant.stackexchange.com')
ProjectManagement = __SEAPI('api.pm.stackexchange.com')
ProjectManagementMeta = __SEAPI('api.meta.pm.stackexchange.com')
Skeptics = __SEAPI('api.skeptics.stackexchange.com')
SkepticsMeta = __SEAPI('api.meta.skeptics.stackexchange.com')
FitnessandNutrition = __SEAPI('api.fitness.stackexchange.com')
FitnessandNutritionMeta = __SEAPI('api.meta.fitness.stackexchange.com')
DrupalAnswers = __SEAPI('api.drupal.stackexchange.com')
DrupalAnswersMeta = __SEAPI('api.meta.drupal.stackexchange.com')
MotorVehicleMaintenanceandRepair = __SEAPI('api.mechanics.stackexchange.com')
MotorVehicleMaintenanceandRepairMeta = __SEAPI('api.meta.mechanics.stackexchange.com')
Parenting = __SEAPI('api.parenting.stackexchange.com')
ParentingMeta = __SEAPI('api.meta.parenting.stackexchange.com')
SharePoint = __SEAPI('api.sharepoint.stackexchange.com')
SharePointMeta = __SEAPI('api.meta.sharepoint.stackexchange.com')
MusicalPracticeandPerformance = __SEAPI('api.music.stackexchange.com')
MusicalPracticeandPerformanceMeta = __SEAPI('api.meta.music.stackexchange.com')
SoftwareQualityAssuranceandTesting = __SEAPI('api.sqa.stackexchange.com')
SoftwareQualityAssuranceandTestingMeta = __SEAPI('api.meta.sqa.stackexchange.com')
JewishLifeandLearning = __SEAPI('api.judaism.stackexchange.com')
JewishLifeandLearningMeta = __SEAPI('api.meta.judaism.stackexchange.com')
GermanLanguageandUsage = __SEAPI('api.german.stackexchange.com')
GermanLanguageandUsageMeta = __SEAPI('api.meta.german.stackexchange.com')
JapaneseLanguageandUsage = __SEAPI('api.japanese.stackexchange.com')
JapaneseLanguageandUsageMeta = __SEAPI('api.meta.japanese.stackexchange.com')
Astronomy = __SEAPI('api.astronomy.stackexchange.com')
AstronomyMeta = __SEAPI('api.meta.astronomy.stackexchange.com')
Philosophy = __SEAPI('api.philosophy.stackexchange.com')
PhilosophyMeta = __SEAPI('api.meta.philosophy.stackexchange.com')
GardeningandLandscaping = __SEAPI('api.gardening.stackexchange.com')
GardeningandLandscapingMeta = __SEAPI('api.meta.gardening.stackexchange.com')
Travel = __SEAPI('api.travel.stackexchange.com')
TravelMeta = __SEAPI('api.meta.travel.stackexchange.com')
PersonalProductivity = __SEAPI('api.productivity.stackexchange.com')
PersonalProductivityMeta = __SEAPI('api.meta.productivity.stackexchange.com')
Cryptography = __SEAPI('api.crypto.stackexchange.com')
CryptographyMeta = __SEAPI('api.meta.crypto.stackexchange.com')
Literature = __SEAPI('api.literature.stackexchange.com')
LiteratureMeta = __SEAPI('api.meta.literature.stackexchange.com')
SignalProcessing = __SEAPI('api.dsp.stackexchange.com')
SignalProcessingMeta = __SEAPI('api.meta.dsp.stackexchange.com')
FrenchLanguageandUsage = __SEAPI('api.french.stackexchange.com')
FrenchLanguageandUsageMeta = __SEAPI('api.meta.french.stackexchange.com')
Christianity = __SEAPI('api.christianity.stackexchange.com')
ChristianityMeta = __SEAPI('api.meta.christianity.stackexchange.com')
Bitcoin = __SEAPI('api.bitcoin.stackexchange.com')
BitcoinMeta = __SEAPI('api.meta.bitcoin.stackexchange.com')
